
import processing.opengl.*;

// variables for rendering
int[] bgColor = {230, 230, 230};
int[] tCol = {0, 0, 0};
int[] tColOff = {180, 180, 180};
int[] netCol = {50, 50, 50};
PFont font;
PFont smallFont;
float scaleSpace = 130;
float trichordSize = 12;
boolean wrapSpace = true; // infinitely wrap the space
boolean multiSets = true; // add multi-sets
boolean permuteEquiv = false; // permutational equivalence on/off
String show = "all";

final boolean PRIMEFORM = true;
final boolean NORMALFORM = false;
boolean setType = PRIMEFORM; // set whether to display prime or normal form (PRIMEFORM, NORMALFORM)

Trichord trichord[];

void setup() {
  size(800, 600, OPENGL); // or P3D
  //hint(ENABLE_DEPTH_SORT); // this REALLY slows down rendering when dealing with so many objects
  float cameraZ = ((height/2.0) / tan(PI*60.0/360.0));
  // 3rd and 4th arguments specify nearclip and farclip of camera in OPENGL
  perspective(PI/3.0, float(width)/float(height), cameraZ/100., cameraZ*10.);
  frameRate(60); // framerate of main window
  consoleFont = loadFont("GaramondPremrPro-48.vlw"); 
  uiWindow = new UIWindow(430, 311, 15, consoleFont, "Console"); // width, height, framerate, font name, window name 

  font = loadFont("SansSerif-48.vlw"); 
  smallFont = loadFont("SansSerif-12.vlw");   
  textAlign(CENTER, CENTER);
  fill(0, 0, 0, 255); 
  genSets(); // define instances of Sphere class
  navSetup(); // this sets up the 3Dnav - place it at the end of your setup() function 
}

void draw() {
  background(bgColor[0], bgColor[1], bgColor[2]);
  drawSets();
  navUpdate(); // this updates the 3Dnav - place it at the end of your draw() function 
  //println(frameRate); 
}


////////////////////////////////////////////////////////
////////custom functions for rendering of space/////////
////////////////////////////////////////////////////////

void genSets() {
  int indexNum = 0;
  trichord = new Trichord[13824];
  
  // build space in 8 12x12 blocks
  for(int i = 0; i > -12; i--) {
    for(int j = 0; j > -12; j--) {
      for(int k = 0; k < 12; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  }
  
  for(int i = -12; i > -24; i--) {
    for(int j = 0; j > -12; j--) {
      for(int k = 0; k < 12; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  } 

  for(int i = 0; i > -12; i--) {
    for(int j = 0; j > -12; j--) {
      for(int k = 12; k < 24; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  } 

  for(int i = -12; i > -24; i--) {
    for(int j = 0; j > -12; j--) {
      for(int k = 12; k < 24; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  } 

  for(int i = 0; i > -12; i--) {
    for(int j = -12; j > -24; j--) {
      for(int k = 0; k < 12; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  } 

  for(int i = -12; i > -24; i--) {
    for(int j = -12; j > -24; j--) {
      for(int k = 0; k < 12; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  } 

  for(int i = 0; i > -12; i--) {
    for(int j = -12; j > -24; j--) {
      for(int k = 12; k < 24; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  } 

  for(int i = -12; i > -24; i--) {
    for(int j = -12; j > -24; j--) {
      for(int k = 12; k < 24; k++) {
        trichord[indexNum] = new Trichord(i, j, k, tCol[0], tCol[1], tCol[2], trichordSize);  
        indexNum++; 
      }
    }
  }   
}



void drawSets() {
  //println(cam);
  if(wrapSpace) {
    for (int i=0; i<trichord.length; i++) {
      trichord[i].display();
    }    
  }
  else { 
    for (int i=0; i<trichord.length/8; i++) {
      trichord[i].display();
    }
  } 
}

