/////////////////////////////////
//////// custom classes /////////
/////////////////////////////////

class Trichord {

  // properties
  float ic1;
  float ic2;
  float ic3;  
  float x;
  float y;
  float z;
  int[] tri = new int[3]; // original trichord
  int[] tri1 = new int[3]; // 3 rotations of trichord (in ascending order)
  int[] tri2 = new int[3];
  int[] tri3 = new int[3];  
  int[] normTri = new int[3];
  int[] primeTri = new int[3];
  int r;
  int g;
  int b;
  float triSize;
  float triSize2;
  float Yrad;
  float Xrad;
  float colr;
  float dst;
  String name;
  String normName;
  String primeName;
  boolean drawText;
  boolean shrinkFont;
  boolean drawLine;
  boolean textOn;
  String drawLineXYZ = "";

  // constructors
  Trichord(float ic1, float ic2, float ic3, int R, int G, int B, float tSize) { // custom constructor
    this.ic1 = ic1*scaleSpace;
    this.ic2 = ic2*scaleSpace;
    this.ic3 = ic3*scaleSpace;
    this.x = ic1*scaleSpace;
    this.y = ic2*scaleSpace;
    this.z = ic3*scaleSpace;
    this.tri[0] = (12 - int(ic1%12)) % 12;
    this.tri[1] = (12 - int(ic2%12)) % 12;
    this.tri[2] = int(ic3%12);     
    this.r = R;
    this.g = G;
    this.b = B;
    this.triSize = tSize;
    this.triSize2 = tSize/2;
    this.drawText = true;
    genName();  
    getRotations();
    getNormalForm();
    getPrimeForm();
    genNormName();
    genPrimeName();

  }

  void genName() {
    String xn = str(this.tri[0]);
    String yn = str(this.tri[1]);
    String zn = str(this.tri[2]);    
    if(xn.equals("10")) xn = "T";
    if(xn.equals("11")) xn = "E";
    if(yn.equals("10")) yn = "T";
    if(yn.equals("11")) yn = "E";
    if(zn.equals("10")) zn = "T";
    if(zn.equals("11")) zn = "E";    
    this.name = xn+" "+yn+" "+zn;   
  }

  void getRotations() { // derive rotations of trichord
    tri1 = sort(tri); // arrange tri1 in ascending order
    tri2[0] = tri1[1];
    tri2[1] = tri1[2];
    tri2[2] = tri1[0];

    tri3[0] = tri1[2];
    tri3[1] = tri1[0];
    tri3[2] = tri1[1];
  }  

  void getNormalForm() {
    int gap1 = tri1[2] - tri1[0];
    if(gap1 < 0) gap1 = 12 + gap1; // invert if < 0
    if(gap1 == 0) gap1 = 100;

    int gap2 = tri2[2] - tri2[0];
    if(gap2 < 0) gap2 = 12 + gap2; // invert if < 0
    if(gap2 == 0) gap2 = 200;  

    int gap3 = tri3[2] - tri3[0];
    if(gap3 < 0) gap3 = 12 + gap3; // invert if < 0  
    if(gap3 == 0) gap3 = 400;   

    if(gap1 < gap2 && gap1 < gap3) normTri = tri1;
    if(gap2 < gap1 && gap2 < gap3) normTri = tri2;
    if(gap3 < gap1 && gap3 < gap2) normTri = tri3;
    if(gap1 < gap3 && gap1 == gap2) {
      int gap1a = tri1[1] - tri1[0];
      if(gap1a < 0) gap1a = 12 + gap1a; // invert if < 0

      int gap2a = tri2[1] - tri2[0];
      if(gap2a < 0) gap2a = 12 + gap2a; // invert if < 0

      if(gap1a < gap2a) normTri = tri1;
      if(gap2a < gap1a) normTri = tri2; 
      if(gap1a == gap2a) {
        if(tri1[0] <= tri2[0]) normTri = tri1;
        else normTri = tri2;
      }   
    }
    if(gap1 < gap2 && gap1 == gap3) {
      int gap1a = tri1[1] - tri1[0];
      if(gap1a < 0) gap1a = 12 + gap1a; // invert if < 0

      int gap3a = tri3[1] - tri3[0];
      if(gap3a < 0) gap3a = 12 + gap3a; // invert if < 0

      if(gap1a < gap3a) normTri = tri1;
      if(gap3a < gap1a) normTri = tri3; 
      if(gap1a == gap3a) {
        if(tri1[0] <= tri3[0]) normTri = tri1;
        else normTri = tri3;
      }    
    }
    if(gap2 < gap1 && gap2 == gap3) {
      int gap2a = tri2[1] - tri2[0];
      if(gap2a < 0) gap2a = 12 + gap2a; // invert if < 0

      int gap3a = tri3[1] - tri3[0];
      if(gap3a < 0) gap3a = 12 + gap3a; // invert if < 0

      if(gap2a < gap3a) normTri = tri2;
      if(gap3a < gap2a) normTri = tri3; 
      if(gap2a == gap3a) {
        if(tri2[0] <= tri3[0]) normTri = tri2;
        else normTri = tri3;
      }      
    }  
    if(gap1 == gap2 && gap1 == gap3) { // if fully symmetric (i.e. 048)
      if(tri1[0] <= tri2[0] && tri1[0] <= tri3[0]) normTri = tri1;
      else if(tri2[0] <= tri1[0] && tri2[0] <= tri3[0]) normTri = tri2;
      else if(tri3[0] <= tri2[0] && tri3[0] <= tri1[0]) normTri = tri3; 
    }   

    //println("normal form: " + normTri[0] + " " + normTri[1] + " " + normTri[2]);
  }

  void getPrimeForm() {
    int[] prime1 = new int[3];
    int[] prime2 = new int[3];  

    prime1[0] = 0;
    prime1[1] = abs(normTri[1] - normTri[0]);
    if(prime1[1] > 6) prime1[1] = 12 - prime1[1];
    int gap1 = abs(normTri[2] - normTri[1]);
    if(gap1 > 6) gap1 = 12 - gap1;  
    prime1[2] = prime1[1] + gap1;

    prime2[0] = 0;
    prime2[1] = abs(normTri[1] - normTri[2]);
    if(prime2[1] > 6) prime2[1] = 12 - prime2[1];
    int gap2 = abs(normTri[0] - normTri[1]);
    if(gap2 > 6) gap2 = 12 - gap2;  
    prime2[2] = prime2[1] + gap2;

    if(prime1[1] <= prime2[1]) primeTri = prime1;
    else primeTri = prime2;

    //println("prime form: " + primeTri[0] + " " + primeTri[1] + " " + primeTri[2]); 
  }

  void genNormName() {
    String xn = str(this.normTri[0]);
    String yn = str(this.normTri[1]);
    String zn = str(this.normTri[2]);     
    if(xn.equals("10")) xn = "T";
    if(xn.equals("11")) xn = "E";
    if(yn.equals("10")) yn = "T";
    if(yn.equals("11")) yn = "E";
    if(zn.equals("10")) zn = "T";
    if(zn.equals("11")) zn = "E";    
    this.normName = "["+xn+", "+yn+", "+zn+"]";   
  }

  void genPrimeName() {
    String xn = str(this.primeTri[0]);
    String yn = str(this.primeTri[1]);
    String zn = str(this.primeTri[2]);    
    if(xn.equals("10")) xn = "T";
    if(xn.equals("11")) xn = "E";
    if(yn.equals("10")) yn = "T";
    if(yn.equals("11")) yn = "E";
    if(zn.equals("10")) zn = "T";
    if(zn.equals("11")) zn = "E";    
    this.primeName = xn+" "+yn+" "+zn;  
  }  

  void setColor(int R, int G, int B) {
    this.r = R;
    this.g = G;
    this.b = B;
  }

  void getRadians(float X, float Y, float Z) {
    float nX = X - cam[0]; // subtract camera location - this establishes camera at (0,0,0) translates ellipse accordingly
    float nY = Y - cam[1];
    float nZ = Z - cam[2];
    this.Yrad = atan2(nZ, -nX) - HALF_PI; // establish rotation around Y axis (OPENGL is in a left-handed coordinate system)
    float hyp = sqrt(pow(nX,2) + pow(nZ,2));
    this.Xrad = atan2(hyp, nY) - HALF_PI; // establish rotation around X axis
  }

  void getDistance(float X, float Y, float Z) {
    this.dst = dist(cam[0], cam[1], cam[2], X, Y, Z);
    float distThresh;
    if((wrapSpace && !multiSets) || (!wrapSpace && multiSets && !permuteEquiv)) distThresh = scaleSpace * 12; 
    else if((!wrapSpace && !multiSets) || (!wrapSpace && permuteEquiv)) distThresh = scaleSpace * 100;
    else distThresh = scaleSpace*9;
    if(this.dst <= scaleSpace*3) {
      shrinkFont = false;
      drawText = true;
    }
    else if(this.dst <= distThresh) {
      shrinkFont = true;
      drawText = true;
    }
    else drawText = false;
    //this.colr = map(((this.z-this.x)+(this.y-this.x)), 0, 21, 0., 1.);
    //this.colr = 1.;
  }

  void drawText(float X, float Y, float Z) {    
    getRadians(X, Y, Z);
    getDistance(X, Y, Z);         
    //fill(this.r*this.colr, this.g*this.colr, this.b*this.colr);
    fill(this.r, this.g, this.b);
    if(drawText) {    
      pushMatrix();
      translate(X, Y, Z);
      rotateY(this.Yrad);
      rotateX(this.Xrad);
      if(!shrinkFont) {
        textFont(font, this.triSize);      
        rotateY(PI); // flip text 180 degrees to display correctly
        text(this.name, 0, 0);
        if(this.triSize > this.triSize2) translate(0, this.triSize*0.75, 0);
        else translate(0, this.triSize2*0.75, 0);        
        textFont(font, this.triSize2);
        if(setType) {
          //String primeTemp = "("+this.primeName+")";
          text("("+this.primeName+")", 0, 0);
        }
        if(!setType) text(this.normName, 0, 0);
      }
      if(shrinkFont) {
        textFont(smallFont, this.triSize);      
        rotateY(PI); // flip text 180 degrees to display correctly
        text(this.name, 0, 0);
        if(this.triSize > this.triSize2) translate(0, this.triSize*0.75, 0);
        else translate(0, this.triSize2*0.75, 0); 
        textFont(smallFont, this.triSize2);
        if(setType) {
          //String primeTemp = "("+this.primeName+")";
          text("("+this.primeName+")", 0, 0);
        }
        if(!setType) text(this.normName, 0, 0);
      }
      popMatrix();      
    }
  }

  void drawAxes(float X, float Y, float Z) {
    String[] members = split(this.name, ' ');
    strokeWeight(1);
    if(members[0].equals("0") && members[1].equals("0")) {
      stroke(100, 100, 255);
      line(X, Y, Z, X, Y, Z+(scaleSpace/2));
      line(X, Y, Z, X, Y, Z-(scaleSpace/2));  
    }
    if(members[0].equals("0") && members[2].equals("0")) {
      stroke(80, 210, 80); 
      line(X, Y, Z, X, Y+(scaleSpace/2), Z);
      line(X, Y, Z, X, Y-(scaleSpace/2), Z); 
    }  
    if(members[1].equals("0") && members[2].equals("0")) {
      stroke(255, 100, 100); 
      line(X, Y, Z, X+(scaleSpace/2), Y, Z);
      line(X, Y, Z, X-(scaleSpace/2), Y, Z); 
    }      
  }  
  
  void drawTransLine(float X, float Y, float Z) {
    strokeWeight(2);
    stroke(netCol[0], netCol[1], netCol[2]);
    float[] temp = float(split(this.drawLineXYZ, ' '));
    for(int i=0; i<=temp.length-3; i+=3) {
      line(X, Y, Z, X-temp[i]*scaleSpace, Y-temp[i+1]*scaleSpace, Z+temp[i+2]*scaleSpace);
    }
  }

  void doTranslate(float X, float Y, float Z) {
    float nX = cam[0] - X; 
    float nY = cam[1] - Y;
    float nZ = cam[2] - Z;
    int multX = int(nX/(scaleSpace*12));
    int multY = int(nY/(scaleSpace*12));
    int multZ = int(nZ/(scaleSpace*12));      
    this.x = this.x + (scaleSpace*24*multX); 
    this.y = this.y + (scaleSpace*24*multY); 
    this.z = this.z + (scaleSpace*24*multZ);      
  }
  
  void resetTranslate() {
    this.x = this.ic1;
    this.y = this.ic2;
    this.z = this.ic3;
  }

  void display() {
    if(wrapSpace) {
      this.doTranslate(this.x, this.y, this.z);
    }     
    //println("x: " + screenX(this.x, this.y, this.z));
    //println("y: " + screenY(this.x, this.y, this.z)); 
    //println("z: " + screenZ(this.x, this.y, this.z)); 
    if(!multiSets) {
      if(this.tri[0] < this.tri[1] && this.tri[1] < this.tri[2]) {
        if(screenZ(this.x,this.y,this.z) > 0 && screenZ(this.x,this.y,this.z) <= 1
          && screenX(this.x,this.y,this.z) > 0 && screenX(this.x,this.y,this.z) < width 
          && screenY(this.x,this.y,this.z) > 0 && screenY(this.x,this.y,this.z) < height) {
          this.drawAxes(this.x, this.y, this.z);       
          this.drawText(this.x, this.y, this.z);   
        }
      }
    }
    else {
      if(!permuteEquiv) {
        if(screenZ(this.x,this.y,this.z) > 0 && screenZ(this.x,this.y,this.z) <= 1
          && screenX(this.x,this.y,this.z) > 0 && screenX(this.x,this.y,this.z) < width 
          && screenY(this.x,this.y,this.z) > 0 && screenY(this.x,this.y,this.z) < height) {
          this.drawAxes(this.x, this.y, this.z);       
          this.drawText(this.x, this.y, this.z);   
        }
      }
      else {
        if(this.tri[0] <= this.tri[1] && this.tri[1] <= this.tri[2]) {
          if(screenZ(this.x,this.y,this.z) > 0 && screenZ(this.x,this.y,this.z) <= 1
            && screenX(this.x,this.y,this.z) > 0 && screenX(this.x,this.y,this.z) < width 
            && screenY(this.x,this.y,this.z) > 0 && screenY(this.x,this.y,this.z) < height) {
            this.drawAxes(this.x, this.y, this.z);       
            this.drawText(this.x, this.y, this.z);  
          } 
        }              
      }
    }
    if(this.drawLine) {
      this.drawTransLine(this.x, this.y, this.z);
    }    
  }
}

